import itertools


def assert_equal_not_none(i1, i2):
    assert i1 is not None
    assert i2 is not None
    assert i1 == i2


def compare_iters(it1, it2):
    for i1, i2 in itertools.zip_longest(it1, it2):
        assert_equal_not_none(i1, i2)



import io
from unittest import mock

import json_streams
import pytest

from tests.utils import compare_iters


@pytest.mark.parametrize(
    "file_name,json_format, expected_iter",
    [
        ("test", "json", "json_iter"),
        ("test", None, "json_iter"),
        ("test.json", None, "json_iter"),
        ("test", "jsonl", "jsonl_iter"),
        ("test.jsonl", None, "jsonl_iter"),
        ("test.jl", None, "jsonl_iter"),
        ("test.gz", "json", "json_iter"),
        ("test.gz", None, "json_iter"),
        ("test.json.gz", None, "json_iter"),
        ("test.gz", "jsonl", "jsonl_iter"),
        ("test.jl.gz", None, "jsonl_iter"),
        ("test.jsonl.gz", None, "jsonl_iter"),
    ],
)
def test_dump_to_file_json(file_name, json_format, expected_iter):
    with mock.patch(
        "json_streams.json_iter.dump_to_file"
    ) as json_iter_mock, mock.patch(
        "json_streams.jsonl_iter.dump_to_file"
    ) as jsonl_iter_mock:
        in_iter = None
        json_streams.dump_to_file(in_iter, file_name, json_format=json_format)
        expected_call = [mock.call(in_iter, file_name, file_mode="wb")]
        if expected_iter == "json_iter":
            assert json_iter_mock.mock_calls == expected_call
            jsonl_iter_mock.assert_not_called()
        else:
            json_iter_mock.assert_not_called()
            assert jsonl_iter_mock.mock_calls == expected_call


@pytest.mark.parametrize(
    "file_name,json_format, expected_iter",
    [
        ("test", "json", "json_iter"),
        ("test", None, "json_iter"),
        ("test.json", None, "json_iter"),
        ("test", "jsonl", "jsonl_iter"),
        ("test.jsonl", None, "jsonl_iter"),
        ("test.jl", None, "jsonl_iter"),
        ("test.gz", "json", "json_iter"),
        ("test.gz", None, "json_iter"),
        ("test.json.gz", None, "json_iter"),
        ("test.gz", "jsonl", "jsonl_iter"),
        ("test.jl.gz", None, "jsonl_iter"),
        ("test.jsonl.gz", None, "jsonl_iter"),
    ],
)
def test_sink_from_file_json(file_name, json_format, expected_iter):
    with mock.patch(
        "json_streams.json_iter.sink_from_file"
    ) as json_iter_mock, mock.patch(
        "json_streams.jsonl_iter.sink_from_file"
    ) as jsonl_iter_mock:
        json_streams.sink_from_file(file_name, json_format=json_format)
        expected_call = [mock.call(file_name, file_mode="wb")]
        if expected_iter == "json_iter":
            assert json_iter_mock.mock_calls == expected_call
            jsonl_iter_mock.assert_not_called()
        else:
            json_iter_mock.assert_not_called()
            assert jsonl_iter_mock.mock_calls == expected_call


@pytest.mark.parametrize(
    "file_name,json_format, expected_iter",
    [
        ("test", "json", "json_iter"),
        ("test", None, "json_iter"),
        ("test.json", None, "json_iter"),
        ("test", "jsonl", "jsonl_iter"),
        ("test.jsonl", None, "jsonl_iter"),
        ("test.jl", None, "jsonl_iter"),
        ("test.gz", "json", "json_iter"),
        ("test.gz", None, "json_iter"),
        ("test.json.gz", None, "json_iter"),
        ("test.gz", "jsonl", "jsonl_iter"),
        ("test.jl.gz", None, "jsonl_iter"),
        ("test.jsonl.gz", None, "jsonl_iter"),
    ],
)
def test_load_from_file_json(file_name, json_format, expected_iter):
    with mock.patch(
        "json_streams.json_iter.load_from_file"
    ) as json_iter_mock, mock.patch(
        "json_streams.jsonl_iter.load_from_file"
    ) as jsonl_iter_mock:
        for _ in json_streams.load_from_file(file_name, json_format=json_format):
            pass
        expected_call = mock.call(file_name, file_mode="rb")
        if expected_iter == "json_iter":
            assert expected_call in json_iter_mock.mock_calls
            jsonl_iter_mock.assert_not_called()
        else:
            json_iter_mock.assert_not_called()
            assert expected_call in jsonl_iter_mock.mock_calls


@pytest.mark.parametrize(
    "data,facit,json_format",
    [
        (1, b"1", "json"),
        (1, b"1\n", "jsonl"),
        ([1, 2], b"[1,2]", "json"),
        ([1, 2], b"1\n2\n", "jsonl"),
    ],
)
def test_dump_int_memoryio(data, facit, json_format):
    out = io.BytesIO()
    json_streams.dump(data, out, json_format=json_format)
    assert out.getvalue() == facit

    out.seek(0)  # read it from the beginning
    out_iter = json_streams.load(out, json_format=json_format)
    if isinstance(data, list):
        compare_iters(out_iter, data)
    else:
        for i in json_streams.load(out, json_format=json_format):
            print("i = {i}".format(i=i))
            assert i == data


@pytest.mark.parametrize(
    "data,facit,json_format",
    [
        (1, b"1", "json"),
        (1, b"1\n", "jsonl"),
        ([1, 2], b"[\n1,\n2\n]", "json"),
        ([1, 2], b"1\n2\n", "jsonl"),
    ],
)
def test_sink_int_memoryio(data, facit, json_format):
    out = io.BytesIO()
    with json_streams.sink(out, json_format=json_format) as sink:
        for d in data if isinstance(data, list) else [data]:
            sink.send(d)
    assert out.getvalue() == facit



"""Handle JSON or JSON_LINES lazily."""
import sys
import typing
from typing import BinaryIO, Iterable, Optional

from json_streams import _types, encoders, json_iter, jsonl_iter, utility

__all__ = ["encoders", "json_iter", "jsonl_iter", "utility"]


# pylint: disable=unsubscriptable-object
def choose_iter(name, json_format: Optional[utility.JsonFormat]):
    if json_format == utility.JsonFormat.JSON_LINES or utility.is_jsonl(name):
        return jsonl_iter
    return json_iter


def load(fp: BinaryIO, *, json_format: Optional[utility.JsonFormat] = None) -> Iterable:
    _iter = choose_iter(utility.get_name_of_file(fp), json_format)

    yield from _iter.load(fp)


def load_from_file(
    file_name: typing.Optional[_types.Pathlike],
    *,
    json_format: Optional[utility.JsonFormat] = None,
    file_mode: str = "rb",
    use_stdin_as_default: bool = False,
    **kwargs,
) -> Iterable:
    """Lazily load from given file_name.

    Reads from stdin if `use_stdin_as_default` is set and file_name is falsy.

    Parameters:
        file_name : Path
            name of file to load from
        json_format : utility.JsonFormat, optional, default=None
            Explicit set format of json file
        file_mode : str, optional, default="rb"
            mode to open the file in.
        use_stdin_as_default : bool, optional, default=False
            reads from stdin if file_name is None

    Yields:
        JSON value
            lazily loaded from file
    """
    if file_name is not None:
        _iter = choose_iter(file_name, json_format)

        yield from _iter.load_from_file(file_name, file_mode=file_mode, **kwargs)

    elif use_stdin_as_default:
        yield from jsonl_iter.load(sys.stdin.buffer, **kwargs)
    else:
        raise ValueError("You can't read from a empty file")


def dump(in_iter_, fp: BinaryIO, *, json_format: Optional[utility.JsonFormat] = None):
    _iter = choose_iter(utility.get_name_of_file(fp), json_format)

    _iter.dump(in_iter_, fp)


def dump_to_file(
    in_iter_,
    file_name: typing.Optional[_types.Pathlike],
    *,
    json_format: Optional[utility.JsonFormat] = None,
    file_mode: str = "wb",
    use_stdout_as_default: bool = False,
    use_stderr_as_default: bool = False,
    **kwargs,
):
    """Open file and dump json to it.

    Parameters:
        in_iter_ : Iterable
            The data to dump.
        file_name : Path
            The path to write to
        json_format : utility.JsonFormat, optional, default=None
            the json format to write in.
        file_mode : str, optional, default="wb"
            the mode to open the file in.
        use_stdout_as_default : bool, optional, default=False
            use stdout if file_name is empty.
        use_stderr_as_default : bool, optional, default=False
            use stderr if file_name is empty.
    """
    if file_name is not None:
        _iter = choose_iter(file_name, json_format)

        _iter.dump_to_file(in_iter_, file_name, file_mode=file_mode, **kwargs)

    elif use_stdout_as_default:
        jsonl_iter.dump(in_iter_, sys.stdout.buffer, **kwargs)
    elif use_stderr_as_default:
        jsonl_iter.dump(in_iter_, sys.stderr.buffer, **kwargs)
    else:
        raise ValueError("file_name can't be empty")


def sink(fp: BinaryIO, *, json_format: Optional[utility.JsonFormat] = None):
    _iter = choose_iter(utility.get_name_of_file(fp), json_format)

    return _iter.sink(fp)


def sink_from_file(
    file_name: typing.Optional[_types.Pathlike],
    *,
    json_format: Optional[utility.JsonFormat] = None,
    file_mode: str = "wb",
    use_stdout_as_default: bool = False,
    use_stderr_as_default: bool = False,
):
    """Open file and use it as json sink.

    Args:
        file_name : Path
            the path to write to
        json_format : utility.JsonFormat, optional, default=None
            the json format to write in.
        file_mode : str, optional, default="wb"
            the mode to open the file in.
        use_stdout_as_default : bool, optional, default=False
            use stdout if file_name is empty.
        use_stderr_as_default : bool, optional, default=False
            use stderr if file_name is empty.
    """
    if not file_name:
        if use_stdout_as_default:
            return jsonl_iter.sink(sys.stdout.buffer)
        if use_stderr_as_default:
            return jsonl_iter.sink(sys.stderr.buffer)
        raise ValueError("file_name can't be empty")

    _iter = choose_iter(file_name, json_format)

    return _iter.sink_from_file(file_name, file_mode=file_mode)



"""Util functions.
"""
import os
from enum import Enum
from typing import Any, Generator, Union

from json_streams import _types

# pylint: disable=unsubscriptable-object


def get_name_of_file(fp: _types.File) -> str:
    return fp.name if hasattr(fp, "name") else ""


def is_jsonl(name: _types.Pathlike) -> bool:
    """Test if a filename is json lines.
    Also returns True for '<stdin>' and '<stdout>'.

    Arguments:
        name {str} -- name to test

    Returns:
        {bool} -- True if this name is jsonl.
    """
    if name in ["<stdin>", "<stdout>"]:
        return True
    base, suffix = os.path.splitext(str(name))
    if suffix == ".gz":
        _, suffix = os.path.splitext(base)
    # print('suffix = {suffix}'.format(suffix=suffix))
    return suffix in [".jsonl", ".jl"]


def is_gzip(name: _types.Pathlike) -> bool:
    """Test if a filename is gzip."""
    _, suffix = os.path.splitext(str(name))
    return suffix == ".gz"


def to_bytes(s: Union[str, bytes, bytearray]) -> Union[bytes, bytearray]:
    """Convert str to bytes, otherwise pass through s

    Args:
        s (Union[str, bytes, bytearray]): the argument to assure bytes

    Returns:
        Union[bytes, bytearray]: the possible converted argument
    """
    return s if isinstance(s, (bytes, bytearray)) else s.encode("utf-8")


class Sink:
    def __init__(self, sink: Generator[None, Any, None]):
        self.sink = sink

    def __enter__(self):
        next(self.sink)
        return self.sink

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.sink.close()


class JsonFormat(str, Enum):
    JSON = "json"
    JSON_LINES = "jsonl"



import gzip
import json
from pathlib import Path

import pytest
from json_streams import json_iter


@pytest.mark.skip()
def test_json_gzip():
    filename = Path("tests/data/objs.json.gz")

    num = 0
    for num, obj in enumerate(json_iter.load_from_file(filename)):  # noqa: B007
        assert isinstance(obj, dict)
    assert num == 2


def test_json_gzip_dump_and_load(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_dump_and_load.json.gz")

    json_iter.dump_to_file(entries, filename)

    loaded_entries = list(json_iter.load_from_file(filename))

    assert loaded_entries == entries


def test_json_gzip_dump_fp(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_dump_fp.json.gz")

    with open(filename, "wb") as fp:
        json_iter.dump(entries, fp)

    with gzip.open(filename) as fp:  # type: ignore
        loaded_entries = json.load(fp)

    assert loaded_entries == entries


def test_json_gzip_load_fp(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_load_fp.json.gz")

    with gzip.open(filename, "wt") as fp:
        json.dump(entries, fp)

    with open(filename, "br") as fp:  # type: ignore
        loaded_entries = list(json_iter.load(fp))  # type: ignore

    assert loaded_entries == entries


def test_json_gzip_sink_and_load(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_sink_and_load.json.gz")

    with json_iter.sink_from_file(filename) as sink:
        for entry in entries:
            sink.send(entry)

    loaded_entries = list(json_iter.load_from_file(filename))

    assert loaded_entries == entries


def test_json_gzip_sink_fp(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_sink_fp.json.gz")

    with open(filename, mode="wb") as fp:
        with json_iter.sink(fp) as sink:
            for entry in entries:  # sourcery skip: no-loop-in-tests
                sink.send(entry)

    loaded_entries = list(json_iter.load_from_file(filename))

    assert loaded_entries == entries



import gzip
import json
from pathlib import Path

from json_streams import jsonl_iter


def test_jsonl_gzip():
    filename = Path("tests/data/objs.jsonl.gz")

    num = 0
    for num, obj in enumerate(jsonl_iter.load_from_file(filename)):  # noqa: B007
        assert isinstance(obj, dict)
    assert num == 2


def test_jsonl_gzip_dump_and_load(entries: list[dict]):
    filename = Path("tests/data/gen/jsonl_gzip_dump_and_load.jsonl.gz")

    jsonl_iter.dump_to_file(entries, filename)

    loaded_entries = list(jsonl_iter.load_from_file(filename))

    assert loaded_entries == entries


def test_jsonl_gzip_dump_fp(entries: list[dict]):
    filename = Path("tests/data/gen/jsonl_gzip_dump_fp.json.gz")

    with open(filename, "wb") as fp:
        jsonl_iter.dump(entries, fp)

    with gzip.open(filename, mode="rt") as fp:  # type: ignore
        loaded_entries = [json.loads(line) for line in fp]

    assert loaded_entries == entries


def test_json_gzip_load_fp(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_load_fp.json.gz")

    with gzip.open(filename, "wt") as fp:
        for entry in entries:
            json.dump(entry, fp)
            fp.write("\n")

    with open(filename, "rb") as fp:  # type: ignore
        loaded_entries = list(jsonl_iter.load(fp))  # type: ignore

    assert loaded_entries == entries


def test_jsonl_gzip_sink_and_load(entries: list[dict]):
    filename = Path("tests/data/gen/jsonl_gzip_sink_and_load.jsonl.gz")

    with jsonl_iter.sink_from_file(filename) as sink:
        for entry in entries:
            sink.send(entry)

    loaded_entries = list(jsonl_iter.load_from_file(filename))

    assert loaded_entries == entries


def test_jsonl_gzip_sink_fp(entries: list[dict]):
    filename = Path("tests/data/gen/jsonl_gzip_sink_fp.jsonl.gz")

    with open(filename, mode="wb") as fp:
        with jsonl_iter.sink(fp) as sink:
            for entry in entries:
                sink.send(entry)

    loaded_entries = list(jsonl_iter.load_from_file(filename))

    assert loaded_entries == entries



import json_streams
import pytest


@pytest.mark.parametrize(
    "dump_kwargs", [{}, {"default": json_streams.encoders.encode_decimal}]
)
@pytest.mark.parametrize(
    "outfile", ["tests/data/gen/array.json", "tests/data/gen/array.jsonl"]
)
@pytest.mark.parametrize("infile", ["tests/data/array.json", "tests/data/array.jsonl"])
def test_streaming(infile, outfile, dump_kwargs):
    json_streams.dump_to_file(
        json_streams.load_from_file(infile), outfile, **dump_kwargs
    )


class TestLoadFromFile:
    def test_behaves_same_for_json_and_jsonl(self) -> None:
        loaded_from_json = list(json_streams.load_from_file("tests/data/array.json"))
        loaded_from_json_lines = list(
            json_streams.load_from_file("tests/data/array.jsonl")
        )

        assert loaded_from_json == loaded_from_json_lines



import io
import json

import pytest
from json_streams import json_iter, jsonl_iter

from tests.utils import compare_iters

DATA = [
    {"a": 1},
    {"a": 2},
]


def gen_data():
    yield from DATA


@pytest.mark.parametrize(
    "it,data",
    [
        (json_iter, DATA[0]),
        (jsonl_iter, DATA[0]),
    ],
)
def test_dump_dict_memoryio(it, data):
    out = io.BytesIO()
    it.dump(data, out)

    out.seek(0)
    for i in it.load(out):
        print("i = {i}".format(i=i))
        assert i == data


@pytest.mark.parametrize(
    "it",
    [
        json_iter,
        jsonl_iter,
    ],
)
def test_dump_array_memoryio(it):
    out = io.BytesIO()
    it.dump(DATA, out)

    out.seek(0)
    compare_iters(it.load(out), DATA)


@pytest.mark.parametrize(
    "it",
    [
        json_iter,
        jsonl_iter,
    ],
)
def test_dump_gen_memoryio(it):
    out = io.BytesIO()

    it.dump(gen_data(), out)
    out.seek(0)
    compare_iters(it.load(out), gen_data())


@pytest.mark.parametrize(
    "it,file_name,facit,file_mode",
    [
        (json_iter, "tests/data/array.json", None, "rb"),
    ],
)
def test_load_file_name(it, file_name: str, facit, file_mode):
    if not facit:
        facit = file_name
    with open(facit, encoding="utf-8") as fp:
        facit_it = json.load(fp)
    test_it = it.load_from_file(file_name, file_mode=file_mode)
    compare_iters(test_it, facit_it)


@pytest.fixture(name="strings")
def fixture_strings():
    return ["a", "b", "c"]


@pytest.fixture
def dicts():
    return [{"a": "a1"}, {"b": ["b1", "b2"]}]


def gen_values(lst):
    yield from lst


@pytest.mark.parametrize(
    "it",
    [
        None,
        "a",
        1,
        {},
        {"a": "test"},
        {"a": "test", "b": {"c": "c1"}},
        {"1": 2},
        ["a", "b"],
    ],
)
def test_json_iter_dumps(it):
    out = io.BytesIO()
    for x in json_iter.dumps(it):
        print(f"x = {x}")
        out.write(x)

    result = json.loads(out.getvalue())

    assert result == it


def test_dumps_gen_strings(strings):
    out = io.BytesIO()
    for x in json_iter.dumps(gen_values(strings)):
        out.write(x)

    result = json.loads(out.getvalue())

    assert result == strings


def test_dumps_gen_list(strings):
    out = io.BytesIO()
    for x in json_iter.dumps(gen_values(strings)):
        out.write(x)

    result = json.loads(out.getvalue())

    assert result == strings


def test_dumps_gen_dict(strings):
    data = {"a": "a", "b": gen_values(strings), "c": "c"}

    out = io.BytesIO()
    for x in json_iter.dumps(data):
        out.write(x)

    result = json.loads(out.getvalue())

    expected = {"a": "a", "b": strings, "c": "c"}
    assert result == expected



import sys

import json_streams


def main():
    """Count number of elements in a array.

    Reads either the given path or from `stdin`.

    Example:
    `echo '{"a":1}\n{"b":2}' | python examples/jq_length.py`
    Should print 2
    """
    path = sys.argv[1] if len(sys.argv) > 1 else None
    length = sum(
        (1 for _ in json_streams.load_from_file(path, use_stdin_as_default=True))
    )
    print(length)


if __name__ == "__main__":
    main()



""" Handle JSON-LINES lazily. """

import contextlib
from typing import Iterable, Union

from json_streams import _types, files, jsonlib, utility


def dump(data: Union[dict, Iterable], fileobj: _types.File, **kwargs):
    fp = files.BinaryFileWrite(fileobj=fileobj)
    if isinstance(data, dict):
        fp.write(jsonlib.dumps(data, **kwargs))
        fp.write(b"\n")
        return

    try:
        for obj in data:
            fp.write(jsonlib.dumps(obj, **kwargs))
            fp.write(b"\n")
    except TypeError:
        fp.write(jsonlib.dumps(data, **kwargs))
        fp.write(b"\n")
    if fp.needs_closing:
        fp.close()


def load(fileobj: _types.File, **kwargs) -> Iterable:
    fp = files.BinaryFileRead(fileobj=fileobj)
    for line in fp.file:
        yield jsonlib.loads(line, **kwargs)


def load_from_file(file_name: _types.Pathlike, *, file_mode: str = "rb", **kwargs):
    with files.open_file(file_name, file_mode) as fp:
        yield from load(fp, **kwargs)  # type: ignore


def dump_to_file(obj, file_name: _types.Pathlike, *, file_mode: str = "wb", **kwargs):
    with files.open_file(file_name, file_mode) as fp:
        dump(obj, fp, **kwargs)  # type: ignore


def sink(fileobj: _types.File):
    fp = files.BinaryFileWrite(fileobj=fileobj)
    return utility.Sink(jsonl_sink(fp, close_file=fp.needs_closing))  # type: ignore


def sink_from_file(file_name: _types.Pathlike, *, file_mode: str = "wb"):
    fp = files.open_file(file_name, file_mode)

    return utility.Sink(jsonl_sink(fp, close_file=True))  # type: ignore


def jsonl_sink(fp: _types.File, *, close_file: bool = False):
    with contextlib.suppress(GeneratorExit):
        while True:
            value = yield
            fp.write(jsonlib.dumps(value))
            fp.write(b"\n")
    if close_file:
        fp.close()



"""
Utility library to load the underlying json library.

Imports `orjson` or `ujson` if either is present, otherwise it imports `json` from
the standard library.
"""
from json_streams import _types, files


def get_backend(backend_name: str):
    """Import the backend named `backend`"""
    import importlib

    return importlib.import_module(f"json_streams.backends.{backend_name}")


def _default_backend():
    for backend_name in ("be_orjson", "be_json"):
        try:
            return get_backend(backend_name)
        except ImportError:
            continue
    raise ImportError("no backends available")


backend = _default_backend()
del _default_backend

dumps = backend.dumps  # type: ignore
loads = backend.loads  # type: ignore


def load_from_file(file_name: _types.Pathlike, file_mode: str = "rb", **kwargs):
    """
    Load the JSON file with the given file_name.

    :param file_name: name of the file to load from.
    :return: the loaded JSON file.
    """
    with files.open_file(file_name, file_mode) as fp:
        return loads(fp.read(), **kwargs)


def dump_to_file(obj, file_name: _types.Pathlike, file_mode: str = "wb", **kwargs):
    """
    Dump to a JSON file with the given file name.

    :param obj: the object to dump.
    :param file_name: name of the file to dump to.
    :return: anything returned from the backend.
    """
    with files.open_file(file_name, file_mode) as fp:
        return fp.write(dumps(obj, **kwargs))



""" Handle JSON lazily. """
from typing import Iterable, Union

import ijson  # type: ignore

from json_streams import _types, files, jsonlib, utility


def dump(data: Union[dict, Iterable], fileobj: _types.File, **kwargs):
    """Dump array to a file object.

    Parameters
    ----------
    data : dict | Iterable
        Iterable object to write.
    fileobj : _types.File
        File object to write to. Must be writable.

    """
    fp = files.BinaryFileWrite(fileobj=fileobj)
    for chunk in dumps(data, **kwargs):
        print(f"writing {chunk=}")
        fp.write(chunk)
    if fp.needs_closing:
        fp.close()


def dumps(obj, **kwargs) -> Iterable[bytes]:
    if isinstance(obj, str):
        yield jsonlib.dumps(obj, **kwargs)
    elif isinstance(obj, dict):
        yield b"{"
        for i, (key, value) in enumerate(obj.items()):
            if i > 0:
                yield b","
            yield b'"%s":' % utility.to_bytes(key)
            yield from dumps(value, **kwargs)
        yield b"}"
    else:
        try:
            it = iter(obj)
        except TypeError:
            yield jsonlib.dumps(obj, **kwargs)
        else:
            yield b"["
            for i, o in enumerate(it):
                if i > 0:
                    yield b","
                yield jsonlib.dumps(o, **kwargs)

            yield b"]"


def load(fileobj: _types.File, *, use_float: bool = True, **kwargs) -> Iterable:
    fp = files.BinaryFileRead(fileobj=fileobj)
    kwargs = {"use_float": use_float} | kwargs
    yield from ijson.items(fp.file, "item", **kwargs)


def load_eager(fp: _types.File):
    data = jsonlib.loads(fp.read())
    if isinstance(data, list):
        yield from data
    else:
        yield data


def load_from_file(file_name: _types.Pathlike, *, file_mode: str = "rb", **kwargs):
    with files.open_file(file_name, file_mode) as fp:
        yield from load(fp, **kwargs)  # type: ignore


def dump_to_file(
    gen: Iterable, file_name: _types.Pathlike, *, file_mode: str = "wb", **kwargs
):
    with files.open_file(file_name, file_mode) as fp:
        return dump(gen, fp, **kwargs)  # type: ignore


def sink(fileobj: _types.File):
    fp = files.BinaryFileWrite(fileobj=fileobj)
    return utility.Sink(json_sink(fp, close_file=fp.needs_closing))  # type: ignore


def sink_from_file(file_name: _types.Pathlike, *, file_mode: str = "wb"):
    fp = files.open_file(file_name, file_mode)
    return utility.Sink(json_sink(fp, close_file=True))  # type: ignore


def json_sink(fp: _types.File, *, close_file: bool = False):
    is_first_value = True
    first_value = None
    try:
        while True:
            value = yield
            if is_first_value and first_value:
                fp.write(b"[\n")
                fp.write(jsonlib.dumps(first_value))
                is_first_value = False
                fp.write(b",\n")
                fp.write(jsonlib.dumps(value))
            elif is_first_value:
                first_value = value
            else:
                fp.write(b",\n")
                fp.write(jsonlib.dumps(value))
    except GeneratorExit:
        if is_first_value and first_value:
            fp.write(jsonlib.dumps(first_value))
        else:
            fp.write(b"\n]")
    if close_file:
        fp.close()



import io

import pytest
from json_streams import json_iter, jsonl_iter

from tests.utils import compare_iters

DATA = [
    {"a": 1},
    {"a": 2},
]


def gen_data():
    yield from DATA


@pytest.mark.parametrize(
    "it,data",
    [
        (json_iter, DATA[0]),
        (jsonl_iter, DATA[0]),
    ],
)
def test_dict_to_sink_memoryio(it, data):
    out = io.BytesIO()
    with it.sink(out) as sink:
        sink.send(data)

    # assert out.getvalue() == facit
    print(f"out = {out.getvalue()}")

    out.seek(0)
    for i in it.load(out):
        print("i = {i}".format(i=i))
        assert i == data


@pytest.mark.parametrize(
    "it",
    [
        json_iter,
        jsonl_iter,
    ],
)
def test_array_to_sink_memoryio(it):
    out = io.BytesIO()
    with it.sink(out) as sink:
        for data in DATA:
            sink.send(data)
    # assert facit == out.getvalue()

    print(f"out = {out.getvalue()}")
    out.seek(0)
    compare_iters(it.load(out), DATA)



import gzip
import json
from pathlib import Path

import json_streams
from json_streams import json_iter, jsonl_iter


def test_jsonl_gzip():
    file = Path("tests/data/objs.jsonl.gz")

    num = 0
    for num, obj in enumerate(json_streams.load_from_file(file)):  # noqa: B007
        assert isinstance(obj, dict)
    assert num == 2


def test_jsonl_gzip_dump_and_load(entries: list[dict]):
    file = Path("tests/data/gen/json_streams_gzip_dump_and_load.jsonl.gz")

    json_streams.dump_to_file(entries, file)

    loaded_entries = list(jsonl_iter.load_from_file(file))

    assert loaded_entries == entries


def test_json_gzip_dump_fp(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_dump_fp.json.gz")

    with open(filename, "wb") as fp:
        json_streams.dump(entries, fp)

    with gzip.open(filename) as fp:  # type: ignore
        loaded_entries = json.load(fp)

    assert loaded_entries == entries


def test_json_gzip_load_fp(entries: list[dict]):
    filename = Path("tests/data/gen/json_gzip_load_fp.json.gz")

    with gzip.open(filename, "wt") as fp:
        json.dump(entries, fp)

    with open(filename, "rb") as fp:  # type: ignore
        loaded_entries = list(json_iter.load(fp))  # type: ignore

    assert loaded_entries == entries


def test_jsonl_gzip_sink_and_load(entries: list[dict]):
    file = Path("tests/data/gen/json_streams_gzip_sink_and_load.jsonl.gz")

    with json_streams.sink_from_file(file) as sink:
        for entry in entries:
            sink.send(entry)

    loaded_entries = list(jsonl_iter.load_from_file(file))

    assert loaded_entries == entries



"""Json backend using standard `json` lib."""
import json
from typing import Any, Union


def dumps(obj, **kwargs) -> bytes:
    return json.dumps(obj, **kwargs).encode("utf-8")


def loads(s: Union[bytes, bytearray, str], **kwargs) -> Any:
    return json.loads(
        s.decode("utf-8") if isinstance(s, (bytes, bytearray)) else s, **kwargs
    )



from json_streams import json_iter


def main():
    data_source = json_iter.load_from_file("benches/data/skbl.json")

    def doc_update(doc):
        doc["_source"]["lexiconName"] = "skbl2"
        doc["_source"]["lexiconOrder"] = 48
        return doc

    update_data = (doc_update(doc) for doc in data_source)
    json_iter.dump_to_file(update_data, "benches/data/skbl2.json")


if __name__ == "__main__":
    main()



import builtins
import gzip
from typing import Any, Optional

from json_streams import _types, utility


def open_file(file_name: _types.Pathlike, mode="rb") -> _types.File:  # type: ignore [type-var]
    assert "b" in mode  # noqa: S101
    if utility.is_gzip(file_name):
        return gzip.GzipFile(file_name, mode)  # type: ignore
    else:
        return builtins.open(file_name, mode)  # type: ignore


class BinaryFile:
    def __init__(
        self,
        mode: str,
        filename: Optional[_types.Pathlike] = None,
        fileobj: Optional[_types.File] = None,
    ) -> None:
        if fileobj is None and filename is None:
            raise ValueError("Must give at least one of 'filename' and 'fileobj'")

        if "t" in mode:
            raise ValueError("Only binary modes supported")
        elif "b" not in mode:
            mode = f"{mode}b"
        self._needs_closing = False
        if fileobj is None:
            self._file = open_file(filename, mode)  # type: ignore
        else:
            fileobj_name = utility.get_name_of_file(fileobj)
            if utility.is_gzip(fileobj_name):
                if isinstance(fileobj, gzip.GzipFile):
                    self._file = fileobj  # type: ignore
                else:
                    self._file = gzip.GzipFile(  # type: ignore
                        filename=filename, fileobj=fileobj, mode=mode
                    )
                    self._needs_closing = True
            else:
                self._file = fileobj  # type: ignore

    @property
    def file(self):
        return self._file

    @property
    def needs_closing(self) -> bool:
        return self._needs_closing

    def close(self):
        if self.needs_closing:
            self._file.close()


class BinaryFileRead(BinaryFile):
    def __init__(
        self,
        filename: Optional[_types.Pathlike] = None,
        fileobj: Optional[_types.File] = None,
    ) -> None:
        super().__init__("rb", filename=filename, fileobj=fileobj)


class BinaryFileWrite(BinaryFile):
    def __init__(
        self,
        filename: Optional[_types.Pathlike] = None,
        fileobj: Optional[_types.File] = None,
        mode: Optional[str] = None,
    ) -> None:
        super().__init__(mode=mode or "wb", filename=filename, fileobj=fileobj)

    def write(self, data: Any) -> int:
        return self._file.write(data)



import pytest
from json_streams.utility import get_name_of_file, is_jsonl


@pytest.mark.parametrize(
    "filename",
    [
        "test.jsonl",
        "test.jl",
        "test.jsonl.gz",
        "test.jl.gz",
    ],
)
def test_filename_is_jsonl(filename: str):
    assert is_jsonl(filename)


@pytest.mark.parametrize(
    "filename",
    [
        "test.json",
        "test.json.gz",
    ],
)
def test_filename_is_not_jsonl(filename: str):
    assert not is_jsonl(filename)


@pytest.mark.parametrize("fp, expected", [(None, "")])
def test_get_name_of_file(fp, expected):
    assert get_name_of_file(fp) == expected



from decimal import Decimal

from json_streams import jsonlib
from json_streams.encoders import encode_decimal


def test_encode_decimal():
    result = jsonlib.dumps([10.20, "10.20", Decimal("10.20")], default=encode_decimal)

    assert result == jsonlib.dumps([10.20, "10.20", 10.20])
